#include "courselib.h"
#include <cmath>
#include "TH1.h"
#include "TFile.h" 
#include <iostream>
#include <string>
#include "LHAPDF/LHAPDF.h"
#include <iostream>
#include <cstdlib>

using namespace std;

/*
   We calculate the momentum sum rule of a parton density function by performing explicitly the integral.
   We use the LHAPDF library to access the different PDF parameterisations.
   We check the momentum sum rule for a standard set, and for the LO* parameterisations.

   We use LHAPDF 6.2.0
   
   Authors: H. Jung, 
            A. Bermudez Martinez, L.I. Estevez Banos, J. Lidrych, M. Mendizabal Morentin, 
            S. Taheri Monfared, P. L.S. Connor, Q. Wang, H. Yang, R. Zlebcik 
   
*/

int main()
{
    const int SUBSET = 0;
      const string NAME = "PB-TMDNLO-HERAI+II-2018-set1";
//    const string NAME = "PB-TMDNLO-HERAI+II-2018-set2";
// 	const string NAME = "MRST2007lomod";		// LO * pdf //
//    const string NAME = "MRSTMCal";			// LO** pdf //
    LHAPDF::initPDFSet(NAME, LHAPDF::LHGRID, SUBSET);

    const int npoints = 100000;


//  initialise random number generator
//  rlxd_init( luxory level, seed )
    rlxd_init(2,32767);

    const double Q = 10.0, mz = 91.2;
    cout << "alphas(mz) = " << LHAPDF::alphasPDF(mz) << endl;
    cout << "qcdlam4    = " << LHAPDF::getLam4(SUBSET) << endl;
    cout << "qcdlam5    = " << LHAPDF::getLam5(SUBSET) << endl;
    cout << "orderPDF   = " << LHAPDF::getOrderPDF() << endl;
    cout << "xmin       = " << LHAPDF::getXmin(SUBSET) << endl;
    cout << "xmax       = " << LHAPDF::getXmax(SUBSET) << endl;
    cout << "q2min      = " << LHAPDF::getQ2min(SUBSET) << endl;
    cout << "q2max      = " << LHAPDF::getQ2max(SUBSET) << endl;
    cout << "orderalfas = " << LHAPDF::getOrderAlphaS() << endl;
    cout << "num flav   = " << LHAPDF::getNf() << endl;
    cout << "name       = " << NAME << endl;
    cout << "number     = " << LHAPDF::numberPDF() << endl;
    cout << endl;


    LHAPDF::initPDF(0);

    const double xmin = LHAPDF::getXmin(0);
    const double xmax = LHAPDF::getXmax(0);

    //      xmax = 0.05 ;
    //      xmin = 0.0001 ;

    double sum0=0, sum00=0;

    for (int n1 = 0; n1 < npoints; ++n1) {
        // for simple integration
        // x = xmin + (xmax-xmin)*Rand();
        // for importance sampling

        double x = xmin * pow((xmax/xmin), Rand());
        double f=0;
        int flavor;
        double ff;
        // the pdf from LHAPDFLIB is called via: xfx(x,Q,flavor) 
        //		with x=fractional momentum
        //		     Q = sqrt(q2), the sqrt of the scale
        //		     flavor = -6,.. ,6 the flavor code of the parton, 
        //		     flavor: 0=gluon, 1=down, 2=up, 3=strange, 4=charm, 5=bottom 6=top           
// insert here your code

// end insert        
        
        sum0  +=  ff;
        sum00 +=  ff*ff; 
        //                                  
    }

    sum0  /= npoints;
    sum00 /= npoints;
    double sigma2 = sum00 - sum0*sum0 ;
    double error = sqrt(sigma2/npoints) ;
    cout<<" mom sum rule is: "<<sum0<<"+/-"<< error<<endl;

	return EXIT_SUCCESS;

}
